#include <MIDI.h>

// Create and bind the MIDI interface to the default hardware Serial port
MIDI_CREATE_DEFAULT_INSTANCE();

int MIDIChannel = 4;

typedef struct {
  int pin;
  int note;
  int state;
} notemap;

// Map pins to MIDI notes and store state
// Using a table instead of simple math just in case I fry an IO pin or something and need to remap
notemap notes[15] = {
// Pin, Note, State
  {0,   0,  0}, //PIN0 is used for MIDI IO
  {1,   0,  0}, //So is PIN1
  {2,   36, 0}, // Note C2
  {3,   35, 0}, // Note B1
  {4,   34, 0}, // https://usermanuals.finalemusic.com/Finale2012Mac/Content/Finale/MIDI_Note_to_Pitch_Table.htm
  {5,   33, 0},
  {6,   32, 0},
  {7,   31, 0},
  {8,   30, 0},
  {9,   29, 0},
  {10,  28, 0},
  {11,  27, 0},
  {12,  26, 0},
  {13,  25, 0},
  {A0,  24, 0}
};

void setup() {
  MIDI.begin(MIDIChannel);

  for (int i=2; i < 16; i++) {
    if (notes[i].note)
      pinMode(notes[i].pin, INPUT_PULLUP);
  }

  pinMode(A1, INPUT_PULLUP);

}

void loop() {
  int transpose = 0;
  
  for (int i=2; i < 16; i++) {
    if (!notes[i].note) continue;
    
    if (!digitalRead(A1))
      transpose = 12;
    else
      transpose = 0;          

    // Switches are normally closed so invert the read
    if (!digitalRead(notes[i].pin)) {      
      if (!notes[i].state) { // Only send one MIDI event per switch action
        MIDI.sendNoteOn(notes[i].note + transpose, 127, MIDIChannel);
        notes[i].state++;
      }
    }
    else {
      if (notes[i].state) {
        MIDI.sendNoteOff(notes[i].note + transpose, 0, MIDIChannel);
        notes[i].state = 0;
      }
    }
  }
  delay(4); // Debounce by waiting 4ms for all pins to settle. We're not in a hurry.
}
